/*\
|*|	console driver interface definition. (See console.c for copyright)
\*/

#ifndef _LINUX_CONSOLE_H
#define _LINUX_CONSOLE_H

#ifdef __KERNEL__
#include <scrdrv/kgi.h>

#define	MAX_NPAR	16

enum attr_fl
{	FL_HALF		= 0x00000000,
	FL_NORMAL	= 0x00000001,
	FL_BOLD		= 0x00000002,
	FL_INTENSITY	= 0x00000003,
	FL_UNDERLINE	= 0x00000004,
	FL_BLINK	= 0x00000008,
	FL_REVERSE	= 0x00000010
};
enum mode_fl
{	FL_DISP_CTRL	= 0x00000001,
	FL_TOGGLE_META	= 0x00000002,
	FL_SCREEN_MODE	= 0x00000004,
	FL_ORIGIN_MODE	= 0x00000008,
	FL_INSERT_MODE	= 0x00000010,
	FL_UTF8_ENCODE	= 0x00000020,
	FL_AUTO_WRAP	= 0x00000040,
	FL_CURSOR_MODE	= 0x00000080,
	FL_POINTER_MODE = 0x00000100
};

struct kgi_vc
{
   ggi_coord	size;			/* logical size			*/
   unsigned int	scr_end;		/* size (in u_shorts)		*/
   short	top, bottom;
   struct kgi_pointer	cursor;
   struct kgi_pointer	pointer;

   ggi_coord	fb_size;		/* framebuffer layout 		*/
   u_short	*fb;			/* pointer to framebuffer	*/
   unsigned int fb_end;			/* size of framebuffer		*/

   void *hwdriver_data;			/* private data for hardware driver */

   unsigned int pos;			/* write position */
   unsigned int ppos;			/* pointer position */
   unsigned int origin;
   int __origin;

   unsigned int	lastpos;		/* 0..lastpos is sync'ed */
   void		(*sync)(struct kgi_vc *);  /* pointer to sync function */
   void		(*set_origin)(unsigned int);

   u_short	erase;		/* Background erase character */
   u_char	attr;		/* Current attributes */
   u_char	def_color;	/* Default colors */
   u_char	color;		/* Foreground & background */
   u_char	ulcolor;	/* Colour for underline mode */
   u_char	halfcolor;	/* Colour for half intensity mode */

   ggi_coord	write;		/* write position		*/
   enum attr_fl attr_flags;		/* attributes in effect		*/

   ggi_coord	s_write;	/* saved write position		*/
   enum attr_fl	s_attr_flags;		/* saved attributes		*/
   u_char		s_color;	/* saved color			*/

   unsigned long npar,par[MAX_NPAR];

   int		state;
   enum mode_fl	mode_flags;

   u_char	*translate;
   unsigned long tab_stop[5];		/* tabstops (160 columns) */ 

   u_char	can_do_color;
   
   int		vc_num;		/* the console number */
   struct wait_queue *paste_wait;

 
   /* misc */
   u_char	ques;
   u_char	need_wrap;
   u_char	has_scrolled;	/* Info for unblank_screen */
   u_char	fb_modified;	/* screen has been modified	*/
   unsigned int	vmalloced;	/* size of memory vmalloc()-ed	*/
   u_char	report_mouse;

   u_char	utf_count;
   unsigned long utf_char;

   u_char 	charset;
   u_char	s_charset;   
   u_char 	G0_charset;
   u_char 	G1_charset;
   u_char 	s_G0_charset;
   u_char 	s_G1_charset;
};

#define	STATE		(cons->state)
#define	X		(cons->write.x)
#define	Y		(cons->write.y)
#define	S_X		(cons->s_write.x)
#define	S_Y		(cons->s_write.y)
#define	SIZE_X		(cons->size.x)
#define	SIZE_Y		(cons->size.y)
#define	FB		(cons->fb)
#define	FB_END		(cons->fb_end)
#define	SCR_END		(cons->scr_end)
#define	ORIGIN		(cons->origin)
#define	__ORIGIN	(cons->__origin)
#define	POS		(cons->pos)
#define	PPOS		(cons->ppos)

#define	HAS_SCROLLED	(cons->has_scrolled)

#define	TOP		(cons->top)
#define	BOTTOM		(cons->bottom)
#define	FB_SIZE_X	(cons->fb_size.x)
#define	FB_SIZE_Y	(cons->fb_size.y)

#define	ERASE		(cons->erase)
#define	ATTR		(cons->attr)
#define	COLOR		(cons->color)
#define	NEED_WRAP	(cons->need_wrap)

#define	MODE_FLAGS	(cons->mode_flags)
#define	DISP_CTRL	(cons->mode_flags & FL_DISP_CTRL)
#define	TOGGLE_META	(cons->mode_flags & FL_TOGGLE_META)
#define	SCREEN_MODE	(cons->mode_flags & FL_SCREEN_MODE)
#define	ORIGIN_MODE	(cons->mode_flags & FL_ORIGIN_MODE)
#define	INSERT_MODE	(cons->mode_flags & FL_INSERT_MODE)
#define	UTF8_ENCODE	(cons->mode_flags & FL_UTF8_ENCODE)
#define	AUTO_WRAP	(cons->mode_flags & FL_AUTO_WRAP)
#define	CURSOR_MODE	(cons->mode_flags & FL_CURSOR_MODE)
#define	POINTER_MODE	(cons->mode_flags & FL_POINTER_MODE)
#define	QUES		(cons->ques)

#define	S_ATTR_FLAGS	(cons->s_attr_flags)
#define	ATTR_FLAGS	(cons->attr_flags)
#define	INTENSITY	(cons->attr_flags & FL_INTENSITY)
#define	UNDERLINE	(cons->attr_flags & FL_UNDERLINE)
#define	BLINK		(cons->attr_flags & FL_BLINK)
#define	REVERSE		(cons->attr_flags & FL_REVERSE)
#define	CAN_DO_COLOR	(cons->can_do_color)

#define	COLOR		(cons->color)
#define	S_COLOR		(cons->s_color)
#define	HALFCOLOR	(cons->halfcolor)
#define	DEF_COLOR	(cons->def_color)
#define	ULCOLOR		(cons->ulcolor)
#define	BACKGROUND	(cons->color & 0xF0)
#define	FOREGROUND	(cons->color & 0x0F)


#define	TAB_STOP	(cons->tab_stop)
#define	UTF_COUNT	(cons->utf_count)
#define	UTF_CHAR	(cons->utf_char)

#define	TRANSLATE	(cons->translate)

#define	NPAR		(cons->npar)
#define	PAR		(cons->par)

#define	SYNC		(cons->sync)
#define	SET_ORIGIN	(cons->set_origin)

#define	VC_NUM		(cons->vc_num)
#define	CHARSET		(cons->charset)
#define	S_CHARSET	(cons->s_charset)
#define	G0_CHARSET	(cons->G0_charset)
#define	S_G0_CHARSET	(cons->s_G0_charset)
#define	S_G1_CHARSET	(cons->s_G1_charset)
#define	G1_CHARSET	(cons->G1_charset)

#define	CURSOR		(cons->cursor)
#define	POINTER		(cons->pointer)
#define	VMALLOCED	(cons->vmalloced)
#define	FB_MODIFIED	(cons->fb_modified)

#define	reverse_video_char(X) (((X) & 0x88) | ((((X) >> 4) | ((X) << 4)) & 0x77))

extern struct tty_driver console_driver;

extern int fg_console, last_console;
extern struct kgi_vc *vcons[MAX_NR_CONSOLES];
extern struct kgi_vc *boot_console;
extern void reset_terminal(struct kgi_vc *cons, int do_clear);
extern void scrollfront(unsigned int lines);
extern void scrollback(unsigned int lines);
extern void change_console(int want);
extern int change_console_tty_ops(struct tty_driver *new_driver,
                                  void (*setup)(int));
extern void reset_console_tty_ops(void);


extern void register_console(void (*proc)(const char *)); 

#endif /* #ifdef __KERNEL__ */

#endif	/* #ifdef _LINUX_CONSOLE_H */
