/*\
|*| ---------------------------------------------------------------------------
|*|	structures and types for event reporting
|*| ---------------------------------------------------------------------------
|*|	Copyright 1996 by Steffen Seeger, [seeger@physik.tu-chemnitz.de]
|*| ---------------------------------------------------------------------------
|*|	DO NOT CHANGE, SCRDRV DEPENDS HEAVILY ON THIS IMPLEMENTATION !!
\*/

#ifndef _EVENT_H
#define _EVENT_H
#include <linux/sched.h>

typedef enum 
{			/* broadcast */
  evNothing,				/* event processed/not valid	*/
  evCommand,				/* do a certain action		*/
  evBroadcast,				/* notification about something */
			/* keyboard  */
  evKeyPress,				/* key pressed			*/
  evKeyRelease,				/* key released			*/
  evKeyRepeat,				/* automatic repeat		*/
  evKeyMap,				/* update state (e.g VC switch)	*/
  evKeyUnmap,
			/* last event defined */
  evLast
} event_type;

#define	MASK(X) em ## X = (1 << ev ## X)

typedef enum
{
  MASK(Nothing),		/* broadcast events */
  MASK(Command),
  MASK(Broadcast),

  MASK(KeyPress),		/* keyboard events  */
  MASK(KeyRelease),
  MASK(KeyRepeat),
  MASK(KeyMap),
  MASK(KeyUnmap),

  MASK(Last)			/* last event defined */
} event_mask;

#undef MASK

struct cmd_event
{
  event_type		type;		/* evCommand or evBroadcast	*/
  unsigned int		time;		/* time when event occured	*/
  
  unsigned int		command;	/* command code			*/
  void			*data; 		/* additional information	*/
};

struct kbd_event
{
  event_type		type;		/* one of the keyboard events	*/
  unsigned int		time;		/* time when event occured	*/

  unsigned short	keycode;	/* number of key pressed	*/
  unsigned short	keysym;		/* action bound to keycode	*/
  unsigned short	shift_state;	/* modifier state		*/
  unsigned short	lock_state;	/* locked modifier state	*/
};

typedef union
{
  int			type;
  struct cmd_event	cmd;		/* evCommand or evBroadcast	*/
  struct kbd_event	kbd;		/* evKey......			*/
} ggi_event;

typedef struct
{
  struct wait_queue	*wait;		/* who is waiting for events    */
  unsigned short	size;		/* of *event (power of 2 events)*/
  unsigned short	read;		/* read position 		*/
  unsigned short	write;		/* write position		*/
  unsigned short	overflow;	/* overflow flag		*/
  event_mask		selected;	/* mask of events selected	*/
  ggi_event		*event;		/* circular buffer of events	*/
} event_buffer;

extern inline int
can_put_event(event_buffer *buf)	/* 0 (FALSE) if buf is full.	*/
{
  return (buf->write - buf->read) < buf->size;
}

extern inline int
can_get_event(event_buffer *buf)	/* 0 (FALSE) if no event in buf	*/	
{
  return buf->write != buf->read;
}

extern inline void
put_event(event_buffer *buf, ggi_event *ev)
{
  if (((1 << ev->type) & buf->selected) && (ev->type != evNothing))
    if (can_put_event(buf))
      {
        wake_up(&(buf->wait));

        buf->event[buf->write & (buf->size-1)] = *ev;
        buf->write++;
      }
     else
      buf->overflow = 1;
}

extern inline void
get_event(event_buffer *buf, ggi_event *ev)
{
  if (can_get_event(buf))
    {
      *ev = buf->event[buf->read & (buf->size-1)];
      if (++(buf->read) == buf->size)
        {
          buf->write -= buf->size;
          buf->read  -= buf->size;
        }
    }
}

extern inline void
select_events(event_buffer *buf, event_mask mask)
{
   buf->selected |= mask & ((1 << evLast) - 1);
}

extern inline void
unselect_events(event_buffer *buf, event_mask mask)
{
   buf->selected &= ~mask & ((1 << evLast) - 1);
}

extern inline void
clear_event(ggi_event *ev)
{
  ev->type = evNothing;
}

extern inline void
wait_event(event_buffer *buf)
{
  sleep_on(&(buf->wait));
}

#endif /* #ifdef _EVENT_H */
