/*\
|*| --------------------------------------------------------------------------
|*|	S3 (Vision964) I/O
|*| --------------------------------------------------------------------------
|*|
|*|	Copyright (C) 1995 by Steffen Seeger, [seeger@physik.tu-chemnitz.de]
|*|
|*|	This program is free software; you can redistribute it and/or modify
|*|	it under the terms of the GNU General Public License as published by
|*|	the Free Software Foundation; either version 2, or (at your option)
|*|	any later version.
|*|
|*|	You should have received a copy of the GNU General Public License
|*|	along with this program; see the file COPYING.  If not, write to
|*|	the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
|*|
|*| ---------------------------------------------------------------------------
|*|
|*|	This code was derived from the following source of information:
|*|	[1]	S3 Vision964 Data Book, (c) 1994 S3 Incorporated
\*/

/*\
|*|	Kernel includes
\*/

#include <asm/io.h>

/*\
|*|     GGI includes
\*/

#include "kgi.h"

#define S3IO_BASE   0x03B0
#define S3IO_REGION 0x03E0-S3IO_BASE

static unsigned short INS1r;
static unsigned short CRTI,CRTD;
static unsigned short FCTRLw;

static void
outbMISC(unsigned char value)
{
  outb(value, MISCw);			/* put value      */
  if (value & MISC_COLOR_IO)		/* adjust IO base */
    { CRTI=CRTIc; CRTD=CRTDc; INS1r=INS1rc; FCTRLw=FCTRLwc; }
   else
    { CRTI=CRTIm; CRTD=CRTDm; INS1r=INS1rm; FCTRLw=FCTRLwm; };
}

static inline void 
outbCRT(unsigned char value, unsigned char reg) 
{ outw((value << 8) | reg, CRTI); }

static inline unsigned char
inbCRT(unsigned char reg)
{ outb(reg, CRTI); return inb(CRTD); }


static inline void
outbSEQ(unsigned char value, unsigned char reg)
{ outw((value << 8) | reg, SEQI); }

static inline unsigned char
inbSEQ(unsigned char reg)
{ outb(reg, SEQI); return inb(SEQD); }


static inline void
outbGRC(unsigned char value, unsigned char reg)
{ outw((value << 8 ) | reg, GRCI); }

static inline unsigned char 
inbGRC(unsigned char reg)
{ outb(reg, GRCI); return inb(GRCD); }


static void
s3_unlock(void)
{
  outbCRT(CR38_MAGIC, 0x38);		/* unlock CR30-CR3F */
  outbCRT(CR39_MAGIC, 0x39);		/* unlock CR40-CRFF */
  /* there are more locks, but this should work after reboot. */
}

static void 
s3_lock(void)
{
  outbCRT(0, 0x38);			/* lock CR30-CR3F */
  outbCRT(0, 0x39);			/* lock CR40-CRFF */
}

/*\
|*|	generalized DAC output functions
\*/

static unsigned short dac_port[4] ={ 0x3C8, 0x3C9, 0x3C6, 0x3C7 };

void
outbDAC(unsigned char value, unsigned char dac_reg)
{
  if (dac_reg & ~0xF)
    printk("%s: warning: DAC register select lines truncated: %x to %x.\n",\
           __FILE__, dac_reg, dac_reg &= 0xF);

  outb(0x55,CRTI); outb((inb(CRTD) & ~CR55_DAC_MASK) | (dac_reg >> 2), CRTD);
  outb(value, dac_port[dac_reg & 0x03]);

  #ifdef DEBUG_IO
    TRACE(printk("DAC_%.1x <- %.2x\n", dac_reg, value));
  #endif
}

unsigned char
inbDAC(unsigned char dac_reg)
{
  #ifdef DEBUG_IO
   register unsigned char foo;
  #endif
  
   if(dac_reg & ~0x0F) 
     printk("%s: warning: DAC register select lines truncated: %x to %x.\n",\
	    __FILE__, dac_reg, dac_reg &= 0x0F);

   outb(0x55,CRTI); outb((inb(CRTD) & ~CR55_DAC_MASK) | (dac_reg >> 2), CRTD);

   #ifdef DEBUG_IO
     foo = inb(dac_port[dac_reg & 0x03]);
     TRACE(printk("DAC_%.1x -> %.2x\n",dac_reg, foo));
     return foo;
   #else
     return inb(dac_port[dac_reg & 0x03]);
   #endif
 }

void
insbDAC(unsigned short dac_reg, void *buf, int count)
{
  if(dac_reg & ~0x0F)
    printk("%s: warning; DAC register select lines truncated: %x to %x.\n",\
           __FILE__, dac_reg, dac_reg &= 0xF);

  outb(0x55, CRTI); outb((inb(CRTD) & ~CR55_DAC_MASK) | (dac_reg >> 2), CRTD);
  insb(dac_port[dac_reg & 0x03], buf, count);
}

void
outsbDAC(unsigned short dac_reg, void *buf, int count)
{
  if(dac_reg & ~0x0F) 
    printk("%s: warning: DAC register select lines truncated: %x to %x.\n",\
           __FILE__, dac_reg, dac_reg &= 0x0F);

  outb(0x55, CRTI); outb((inb(CRTD) & ~CR55_DAC_MASK) | (dac_reg >> 2), CRTD);
  outsb(dac_port[dac_reg & 0x03], buf, count);
}

/*\
|*|	clock I/O functions
\*/

void
setCLK(int value)
{
  if(value & ~CR42_DCLK_MASK)
    printk("%s: warning: clock select lines truncated: %x to %x.\n",\
           __FILE__, value, value &= CR42_DCLK_MASK);
  outbCRT((inbCRT(0x42) & ~CR42_DCLK_MASK) | value, 0x42);
}

int
getCLK(void)
{
  return inbCRT(0x5C) & CR5C_CLOCK_MASK;
}


#define NoSEQRegs	0x05
#define	NoCRTRegs	0x80
#define NoGRCRegs	0x09
#define NoATCRegs	0x15

struct statetable
{
   unsigned char MISC;
   unsigned char FCTRL;
   unsigned char SEQ[NoSEQRegs];
   unsigned char CRT[NoCRTRegs];
   unsigned char GRC[NoGRCRegs];
   unsigned char ATC[NoATCRegs];
};

#ifdef NEED_SAVE_STATE
static void
save_SEQ(unsigned char *buf)
{
  int i;

  for(i = 0; i < NoSEQRegs; i++)
   {
     outb(i, SEQI); *(buf++)=inb(SEQD);
     #ifdef DEBUG_IO
       if (!(i & 0xF))
         printk("\n%.2x: ",i);
       printk("%.2X, ",*(buf-1));
     #endif
   }
  #ifdef DEBUG_IO
    printk("\n");
  #endif
}
#endif

static void
restore_SEQ(unsigned char *buf)
{ register unsigned char i;
  for(i=0; i < NoSEQRegs; i++)
   { outb(i,SEQI); outb(*(buf++), SEQD);}
}

#ifdef NEED_SAVE_STATE
static void
save_CRT(unsigned char *buf)
{
  int i;

  for(i = 0; i < NoCRTRegs; i++)
   { 
     outb(i, CRTI); *(buf++)=inb(CRTD);
     #ifdef DEBUG_IO
       if (!(i & 0xF))
         printk("\n%.2x: ",i);
       printk("%.2X, ",*(buf-1));
     #endif
   }
  #ifdef DEBUG_IO
    printk("\n");
  #endif
}
#endif

static void
restore_CRT(unsigned char buf[])
{
  register unsigned char i;

  outbCRT(inbCRT(CRT_VSYNCEND) & ~CR11_LOCKTIMING, CRT_VSYNCEND);
  for(i = 0x00; i < 0x19; i++) outbCRT(buf[i], i); 
  for(i = 0x30; i < 0x3D; i++) outbCRT(buf[i], i);
  for(i = 0x40; i < 0x6E; i++)
    switch(i) 
     {
       case 0x42:
	 outbCRT((inbCRT(i) & CR42_DCLK_MASK) | (buf[i] & ~CR42_DCLK_MASK), i); 
	 break;
       case 0x55:
	 outbCRT((inbCRT(i) & CR55_DAC_MASK) | (buf[i] & ~CR55_DAC_MASK), i);
         break;
       default: 
         outbCRT(buf[i], i);
     }
}

#ifdef NEED_SAVE_SATE
static void
save_GRC(unsigned char *buf)
{
  int i;

  for(i = 0; i < NoGRCRegs; i++) 
   {
     outb(i, GRCI); *(buf++)=inb(GRCD);
     #ifdef DEBUG_IO
       if (!(i & 0xF))
         printk("\n%.2x: ",i);
       printk("%.2X, ",*(buf-1));
     #endif
   }
  #ifdef DEBUG_IO
    printk("\n");
  #endif
}
#endif

static void
restore_GRC(unsigned char *buf)
{
  register unsigned char i;

  for(i = 0; i < NoGRCRegs; i++)
    {
      outb(i, GRCI); outb(*(buf++), GRCD);
      #ifdef DEBUG_IO
        if(!(i & 0xF))
          printk("\n%.2x: ",i);
        printk("%.2X, ",*(buf-1));
      #endif
    }
  #ifdef DEBUG_IO
    printk("\n");
  #endif
}

#ifdef NEED_SAVE_STATE
static void
save_ATC(unsigned char *buf)
{
  int i;

  for(i = 0; i < NoATCRegs; i++)
   { 
     inb(INS1r); outb(i, ATCI); /* reset AFF and setup index */
     *(buf++)=inb(ATCDr);
     #ifdef DEBUG_IO
       if (!(i & 0xF))
         printk("\n%.2x: ",i);
       printk("%.2X, ",*(buf-1));
     #endif
   }
  inb(INS1r); outb( ATCI_ENABLEDISPLAY, ATCI);
}
#endif

static void
restore_ATC(unsigned char *buf)
{
  register unsigned char i;
  /* inb(INS1r);	 initialize AFF */
  for(i = 0; i < NoATCRegs; i++)
   { 
     inb(INS1r); outb(i,ATCI); outb(*(buf++),ATCDw);
     #ifdef DEBUG_IO
       if(!(i & 0x0F))
         printk("\n%.2x: ",i);
       printk("%.2X, ",*(buf-1));
     #endif
   }
  inb(INS1r); outb(ATCI_ENABLEDISPLAY, ATCI);
}

#ifdef NEED_SAVE_STATE
static void
save_state(struct statetable *state)
{
  s3_unlock();
  state->MISC  = inb(MISCr);	/* save miscellaneous output reg */
  #ifdef DEBUG_IO
    printk("MISC: %.2X\n",state->MISC);
  #endif
  state->FCTRL = inb(FCTRLr);	/* save feature control reg */
  save_SEQ(state->SEQ);	/* save sequencer regs */
  save_CRT(state->CRT);	/* save cathode ray tube controller regs */
  save_GRC(state->GRC);	/* save graphics controller regs */
  save_ATC(state->ATC);	/* save attribute controller regs */
}
#endif

static void
restore_state(struct statetable *state)
{
   outbMISC((inb(MISCr) & MISC_CLOCK_MASK) | state->MISC);
   outb(state->FCTRL, FCTRLw);
   restore_SEQ(state->SEQ);
   restore_CRT(state->CRT);
   restore_GRC(state->GRC);
   restore_ATC(state->ATC);
}

#ifdef DEBUG_IO
static void
dump_state(struct statetable *state)
{ int i;
  printk("%.2x\n",state->MISC);
  printk("%.2x\n",state->FCTRL);
  for(i = 0; i < NoSEQRegs; i++)
   { if(!(i & 0xF)) printk("\n%.2x: ",i); printk("%.2x, ",state->SEQ[i]); }
  for(i = 0; i < NoCRTRegs; i++)
   { if(!(i & 0xf)) printk("\n%.2x: ",i); printk("%.2x, ",state->CRT[i]); }
  for(i = 0; i < NoGRCRegs; i++)
   { if(!(i & 0xF)) printk("\n%.2x: ",i); printk("%.2x, ",state->GRC[i]); }
  for(i = 0; i < NoATCRegs; i++)
   { if(!(i & 0xF)) printk("\n%.2x: ",i); printk("%.2x, ",state->ATC[i]); }
}
#endif

