/*\
|*| ---------------------------------------------------------------------------
|*|	generic fixed clock chip driver
|*| ---------------------------------------------------------------------------
|*|
|*|	Copyright (C) 1995 by Steffen Seeger [seeger@physik.tu-chemnitz.de]
|*|
|*|	This program is free software; you can redistribute it and/or modify
|*|	it under the terms of the GNU General Public License as published by
|*|	the Free Software Foundation; either version 2, or (at your option)
|*|	any later version.
|*|
|*|	You should have received a copy of the GNU General Public License
|*|	along with this program; see the file COPYING.  If not, write to
|*|	the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
|*| ---------------------------------------------------------------------------
\*/

#define	MAINTAINER	"Steffen Seeger, [seeger@physik.tu-chemnitz.de]"

#if !(CLOCKVERSION == 01) 
 #warning	kgi_clock data structures changed, check this code !!
#endif

#define	FREQ	clock.clock.freq
#define	INVALIDCLK	-1

/*\
|*|     GGI clock driver interface
\*/

int
kgi_clock_init (void)
{
  TRACE(printk("%s %s fixed clock driver.\n", clock.manufact, clock.model));
  return clock_init();
}

void
kgi_clock_done(void)
{
  clock_done();
  TRACE(printk("%s %s clock driver removed.\n", clock.manufact, clock.model));
}

struct kgi_clock
*kgi_clock_get_info (void)
{
  return &clock;
}

int
kgi_clock_check_mode (struct kgi_mode *mode, enum kgi_tm_cmd cmd)
{
  int i = 0;
  int newfreq = (cmd == CMD_PROPOSE) 
                     ? 0x7FFFFFFF : mode->dclk * mode->clk.mul / mode->clk.div;

  while((FREQ[i] > newfreq) && (i < MAX_FIXED_CLOCKS))
    i++; 			/* search 1st FREQ[i] <= newfreq	*/

  switch (cmd)
   {
     case CMD_PROPOSE:
	mode->clock = INVALIDCLK;
	return EOK;

     case CMD_LOWER:
	if (i == MAX_FIXED_CLOCKS)
	  return -ECLOCK;

	mode->clock = i;
	mode->dclk = FREQ[i] * mode->clk.div / mode->clk.mul;
	TRACE(printk("%s: request for %i Hz, did %i Hz\n", \
	             __FILE__, newfreq, FREQ[i]));
	return EOK;

     case CMD_RAISE:
	if (i == 0)
	  return -ECLOCK;
	 else
          i--;

	mode->clock = i;
	mode->dclk = FREQ[i] * mode->clk.div / mode->clk.mul;

	TRACE(printk("%s: request for %i Hz, did %i Hz\n",
	             __FILE__, newfreq, FREQ[i]));
	return EOK;

     case CMD_CHECK:
	if ((newfreq - FREQ[i]) < (newfreq*2/100))
          {
	    mode->clock = i;
	    return EOK;
          }
	 else
	  {
	    mode->clock = INVALIDCLK; 
	    return -ECLOCK;
	  }

     default:			/* this should not happen ... */
	INTERNALERROR;
	return -ECLOCK;
   }
}

void
kgi_clock_set_mode (struct kgi_mode *mode)
{
  if (mode->clock == INVALIDCLK)
    INTERNALERROR;

  setCLK(index[mode->clock]);
}

int
kgi_clock_ioctl(struct inode *inode, struct file *file, unsigned int cmd,
	       unsigned long arg)
{
  return -EINVAL;
}
