#include <stdio.h>
#include <fcntl.h>
#include <errno.h>

#include <linux/events.h>

#include "graphdev.h"

void 
init_textmode(ggi_mode* mode, int x, int y, int fontx, int fonty)
{
  mode->visible.x = mode->virtual.x = x * fontx; 
  mode->visible.y = mode->virtual.y = y * fonty;
  mode->size.x = mode->size.y = 0;
  mode->textgrid.x = fontx; mode->textgrid.y = fonty;
  mode->graphtype = GT_TEXT;
}

ggi_mode mode;

void 
printf_mode(ggi_mode *mode)
{
  printf("mode = \n{\n\t{ %i, %i },\t/* visible\t*/\n", mode->visible.x, mode->visible.y);
  printf("\t{ %i, %i },\t/* virtual\t*/\n",mode->virtual.x, mode->virtual.y);
  printf("\t{ %i, %i },\t/* size[mm]\t*/\n",mode->size.x, mode->size.y);
  printf("\t%i,\t\t/* graphtype\t*/\n", mode->graphtype);
  printf("\t{ %i, %i }\t/* textgrid\t*/\n};\n",mode->textgrid.x, mode->textgrid.y);
}

static int ggi_fd = -1;

void ggi_init (void)
{
  if ( (ggi_fd = open ("/dev/graphics", O_RDWR)) < 0)
    {				/* mhmm, something went seriously wrong...    */
      perror(NULL);		/* so print the error message and shutdown.   */
      exit(-1);
    }
 				/* file is ok, but is the scrdrv driver?      */
  if (ioctl (ggi_fd, DRIVER_TESTDRIVER, 0) != DEVICEFILE_MAGIC)
    {
      perror("kgi module magic does not match.\n");
      close(ggi_fd);
      exit(-1);
    }				/* magic numbers fit, thus assume all is ok.  */

  /* !!! lib initialization has to be done ... */
}

void ggi_done (void)
{
  if (ggi_fd > 0)
    close(ggi_fd);
}

#define	GGI_IOCTL(C,A)	ioctl(ggi_fd, C, A)

int
ggi_testmode (ggi_mode *mode)
{					/* test if mode is possible */
  return GGI_IOCTL(DRIVER_TESTMODE, mode);
}

int
ggi_set_graphmode (ggi_mode *mode)
{					/* set graphics mode, no mode switch */
  return GGI_IOCTL(DRIVER_SETGRAPHMODE, mode);
}

int
ggi_get_graphmode (ggi_mode *mode)
{					/* get graphics mode		     */
  return GGI_IOCTL(DRIVER_GETGRAPHMODE, mode);
}

int
ggi_graphics (void)
{					/* switch to graphic mode	     */
  return GGI_IOCTL(DRIVER_GRAPHICS, 0);
}

int
ggi_set_textmode (ggi_mode *mode)
{					/* set text mode, no mode switch     */
  return GGI_IOCTL(DRIVER_SETTEXTMODE, mode);
}

int
ggi_get_textmode (ggi_mode *mode)
{					/* get text mode		     */
  return GGI_IOCTL(DRIVER_GETTEXTMODE, mode);
}

int
ggi_text (void)
{					/* switch to text mode		     */
  return GGI_IOCTL(DRIVER_TEXT, 0);
}

int
ggi_wait_event (void)
{
  fd_set read_set;

  FD_ZERO(&read_set); FD_SET(ggi_fd, &read_set);
  while ( ! (select(FD_SETSIZE, &read_set, NULL, NULL,  NULL) > 0));
}

void
print_info(void)
{
  struct kgi_monitor mon;
 
  if (ioctl(ggi_fd, MONITOR_GETINFO, &mon))
    {
      perror("ioctl(MONITOR_GETINFO) not supported by this monitor driver");
      exit(1);
    }

  printf("monitor: %s %s\n",	mon.manufact, mon.model);
  printf("\t%ix%i pixels, ",	mon.maxdots.x, mon.maxdots.y);
  printf("%ix%i mm\n",		mon.size.x, mon.size.y);
  printf("\t%.1f-%.1f MHz, ",	mon.dclk.min*1e-6, mon.dclk.max*1e-6);
  printf("%.1f-%.1f kHz, ",	mon.hfreq.min*1e-3, mon.hfreq.max*1e-3);
  printf("%i-%i Hz\n",		mon.vfreq.min, mon.vfreq.max);
}


main()
{
  ggi_init ();

#if 0

  init_textmode (&mode, 132,50,  9,16);

  if (ggi_testmode (&mode))
    {
      printf ("mode not ok.\n");
    }

  printf("mode ok.\n");

  printf_mode (&mode);
#endif
  print_info();

  ggi_graphics();

  while (1)
    {
      ggi_event ev;

      read(ggi_fd, &ev, sizeof(ev));
        
      printf("type = %i, time = %i, keycode = %.2x, "
	     "keysym = %.4x, shift = %.1x, lock = %.1x\n",
             ev.kbd.type,   ev.kbd.time,        ev.kbd.keycode, 
             ev.kbd.keysym, ev.kbd.shift_state, ev.kbd.lock_state);
    }
}

