/*\
|*| ---------------------------------------------------------------------------
|*|	monitor driver for multisync monitors
|*| ---------------------------------------------------------------------------
|*|
|*|	Copyright (C) 1995 Steffen Seeger, [seeger@physik.tu-chemnitz.de]
|*|
|*|	This program is free software; you can redistribute it and/or modify
|*|	it under the terms of the GNU General Public License as published by
|*|	the Free Software Foundation; either version 2, or (at your option)
|*|	any later version.
|*|
|*|	You should have received a copy of the GNU General Public License
|*|	along with this program; see the file COPYING.  If not, write to
|*|	the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
|*|
|*| ---------------------------------------------------------------------------
\*/

#define	MAINTAINER	"Steffen Seeger [seeger@physik.tu-chemnitz.de]"
#undef DEBUG

#ifdef DEBUG
#define NEED_MONITOR_SYMS
#endif

#include "graphdev.h"
#include "kgi.h"

#define MON_VERSION "0.12"

#if !((MONITORVERSION == 01) && (MODEVERSION == 03))
  #warning kgi data structures changed, check this code !!
#endif

extern struct kgi_monitor monitor;

/*\
|*|	GGI interface
\*/

int
kgi_monitor_init(void)
{
   if (!(monitor.synctype & MULTISYNC))
     {
        printk("error: The monitor configured is not of type MULTISYNC!\n");
     }

   TRACE(printk("%s %s Monitor Driver v"MON_VERSION"\n", \
         monitor.manufact, monitor.model));
   TRACE(printk("(%s - %s, %dx%d Pixel, %d mm x %d mm, %d-%d kHz, %d-%d Hz)\n",\
	  DISPLAYTYPE(monitor.disptype),\
	  (monitor.disptype & MONO_DISPLAY) ? "monochrome" : "color",\
	  monitor.maxdots.x,\
	  monitor.maxdots.y,\
	  monitor.size.x,\
	  monitor.size.y,\
	  monitor.hfreq.min/1000 ,monitor.hfreq.max/1000,\
	  monitor.vfreq.min      ,monitor.vfreq.max ));
   return EOK;
}

void
kgi_monitor_done(void)
{
   TRACE(printk("%s %s Monitor driver removed.\n", \
         monitor.manufact, monitor.model));
}

struct kgi_monitor
*kgi_monitor_get_info (void)
{
  return &monitor;
}

int
kgi_monitor_check_mode (struct kgi_mode *mode, enum kgi_tm_cmd *cmd)
{ 
  int hfreq, vfreq, result;
  register int width = mode->x.width;

  switch (*cmd)
   {
     case CMD_PROPOSE:
	mode->dclk = monitor.dclk.max;

	for(hfreq = monitor.hfreq.max; hfreq > monitor.hfreq.min;
	    mode->dclk -= 1000)
	  {
	    mode->x.syncstart = width * (width      + 41360) /40000;
            mode->x.syncend   = width * (width*7/10 +  8952) / 8000;
            mode->x.total     = width * (width      +  9360) / 8000;
	    mode->x.blankstart= mode->x.syncstart;
	    mode->x.blankend  = mode->x.syncend;

            hfreq = mode->dclk / mode->x.total;
            if (monitor.hfreq.max < hfreq)
              continue;

            mode->y.syncstart = mode->y.width + 3;
            mode->y.syncend   = mode->y.width + 9;
            mode->y.total     = mode->y.width + hfreq/1000 + 1;/*1ms flyback*/
	    mode->y.blankstart= mode->y.syncstart; 
	    mode->y.blankend  = mode->y.syncend;

            mode->x.polarity  = mode->y.polarity = 1; /* doesn't matter */

            vfreq = mode->dclk / (mode->x.total * mode->y.total);

            if ((monitor.vfreq.min <= vfreq) && (vfreq <= monitor.vfreq.max))
              {
	        if (monitor.flags & FL_MON_PROPSIZE)
	          {
 	            mode->request.size.x = mode->x.width * monitor.size.x /
	                                                   monitor.maxdots.x;
	            mode->request.size.y = mode->y.width * monitor.size.y /
	                                                   monitor.maxdots.y;
                  }
	         else
	          mode->request.size = monitor.size;

		TRACE(\
                 printk("%s: calculated %i Hz refresh (%i Hz, DCLK = %i Hz)\n",\
		         __FILE__, vfreq, hfreq, mode->dclk));

		*cmd = CMD_LOWER;
		return EOK;
              }
          }
	TRACE(printk("%s: invalid mode request.\n",__FILE__));
	return -EMONITOR;
	break;

     case CMD_LOWER:
	if (mode->dclk < monitor.dclk.min)
	  return -EMONITOR;

	*cmd = CMD_CHECK;
	return EOK;

     case CMD_RAISE:
	if (mode->dclk > monitor.dclk.max)
	  return -EMONITOR;

	*cmd = CMD_CHECK;
	return EOK;

     case CMD_CHECK: 
	hfreq = mode->dclk / mode->x.total;
	vfreq = mode->dclk / (mode->x.total * mode->y.total);
	
	result =   (monitor.hfreq.min <= hfreq) && (hfreq <= monitor.hfreq.max) 
	        && (monitor.vfreq.min <= vfreq) && (vfreq <= monitor.vfreq.max)
		&& (monitor.dclk.min  <= mode->dclk) 
		&& (mode->dclk <= monitor.dclk.max);

	#ifdef DEBUG
	  if (result)
 	    TRACE(printk("%s: accepted %i Hz refresh (%i Hz, DCLK = %i Hz)\n",\
		__FILE__, vfreq, hfreq, mode->dclk));
	   else
	    TRACE(printk("%s: refresh check failed. (%i Hz, %i Hz, %i Hz)\n",\
		  __FILE__, vfreq, hfreq, mode->dclk));
	#endif

	*cmd = CMD_READY;
	return result ? EOK : -EMONITOR;
	break;

     default:
	INTERNALERROR;
	return -EMONITOR;
   }
}

void
kgi_monitor_set_mode(struct kgi_mode *mode)
{ 
}

int
kgi_monitor_ioctl (struct inode *inode, struct file *file, unsigned int cmd,
		  unsigned long arg)
{
  switch(cmd)
   {
     case MONITOR_GETINFO:
	_out_buf = &monitor; _io_cnt = sizeof(monitor);
	return EOK;
     case MONITOR_SETINFO:
	if (suser())
	  {
	    memcpy(&monitor, _io_buf, sizeof(monitor));
	    return EOK;
	  }
	 else
	  return -EPERM;
     default:
	return -EMONITOR;
   }
}
