/*\
|*| --------------------------------------------------------------------------
|*|	generic timing list driver
|*| --------------------------------------------------------------------------
|*|
|*|	Copyright (C) 1995, The SCRDRV project.
|*|
|*|	This program is free software; you can redistribute it and/or modify
|*|	it under the terms of the GNU General Public License as published by
|*|	the Free Software Foundation; either version 2, or (at your option)
|*|	any later version.
|*|
|*|	You should have received a copy of the GNU General Public License
|*|	along with this program; see the file COPYING.  If not, write to
|*|	the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
|*|
|*| ---------------------------------------------------------------------------
\*/

#define	MAINTAINER	"Steffen Seeger, [seeger@physik.tu-chemnitz.de]"
#define	MON_VERSION	"0.00 ALPHA"

#ifdef DEBUG
  #define NEED_MONITOR_SYMS
#endif

#include "timelist.h"

#if !((MONITORVERSION == 01) && (MODEVERSION == 03))
  #warning kgi data structures changed, check this code!
#endif 

extern struct kgi_monitor monitor;
extern struct kgi_preset timing[MAX_TIMINGS]; 

static int
find_timing(int xwidth, int ywidth)
{
  register int i = -1, found = 0;

  while (i < MAX_TIMINGS && ! found)
   {
     i++;
     found = timing[i].dclk && timing[i].x.width == xwidth 
                            && timing[i].y.width == ywidth;
   }
  return i;
} 

static int 
set_timing(struct kgi_preset *mode)
{
  int i = 0, found = 0;

  if (! mode->dclk) 
    return -EINVAL;
 
  found =  (i = find_timing (mode->x.width, mode->y.width)) < MAX_TIMINGS;

  if (! found)
    {				/* no entry to be replaced		*/
      i = 0;			/* thus search for a free slot		*/
      while (i < MAX_TIMINGS && ! (found = ! timing[i].dclk));
    }

  if (! found)
    return -ENOMEM;

  timing[i] = *mode;
  return EOK;
}

static int
get_timing(struct kgi_preset *mode)
{
  int i = find_timing (mode->x.width, mode->y.width);

  if (i == MAX_TIMINGS)
    return -EINVAL;
   else
    {
      _out_buf = &(timing[i]);
      _io_cnt  = sizeof(timing[i]);
      return EOK; 
    }
}

static int
clr_timing(struct kgi_preset *mode)
{
  int i = find_timing (mode->x.width, mode->y.width);
  if (i != MAX_TIMINGS)
    memset (&(timing[i]), 0, sizeof(timing[i]));
  return EOK;
}

/*\
|*|	GGI interface
\*/

int
kgi_monitor_init(void)
{
  int i;

  TRACE(printk("%s %s monitor driver v" MON_VERSION "\n", \
        monitor.manufact, monitor.model));
  TRACE(printk("(%s - %s, %dx%d Pixel, %d mm x %d mm, %d-%d kHz, %d-%d Hz)\n",\
	  DISPLAYTYPE(monitor.disptype),\
	  (monitor.disptype & MONO_DISPLAY) ? "monochrome" : "color",\
	  monitor.maxdots.x,\
	  monitor.maxdots.y,\
	  monitor.size.x,\
	  monitor.size.y,\
	  monitor.hfreq.min/1000, monitor.hfreq.max/1000, \
	  monitor.vfreq.min,      monitor.vfreq.max ));

  TRACE(printk("modes: "));
  for (i = 0; i < MAX_TIMINGS; i++)
   if (   timing[i].dclk 
       && (monitor.dclk.min <= timing[i].dclk) 
       && (timing[i].dclk <= monitor.dclk.max) )
     TRACE(printk("%ix%i ", timing[i].x.width, timing[i].y.width));
  printk("\n");
  return EOK;
}

void
kgi_monitor_done(void)
{
   TRACE(printk("%s %s monitor driver removed.\n", \
         monitor.manufact, monitor.model));
}

struct kgi_monitor
*kgi_monitor_get_info (void)
{
  return &monitor;
}

int
kgi_monitor_check_mode (struct kgi_mode *mode, enum kgi_tm_cmd *cmd)
{
  int hfreq, vfreq, h, i = 0;
  struct kgi_mode tmp = *mode;
  static struct kgi_mode save;

  if ((i = find_timing (mode->x.width, mode->y.width)) == MAX_TIMINGS)
    return -EMONITOR;				/* nothing found 	      */

  switch (*cmd)
   {
     case CMD_PROPOSE:				/* enter timing found	      */
	mode->dclk = timing[i].dclk;
	mode->x    = timing[i].x;
	mode->y    = timing[i].y;

	if (monitor.flags & FL_MON_PROPSIZE)
	  {
 	    mode->request.size.x = mode->x.width * monitor.size.x /
	                                           monitor.maxdots.x;
	    mode->request.size.y = mode->y.width * monitor.size.y /
	                                           monitor.maxdots.y;
          }
	 else
	  mode->request.size = monitor.size;

	TRACE(printk("%s: propose dclk = %i Hz hfreq = %i Hz vfreq = %i Hz\n",\
	             __FILE__, mode->dclk, mode->dclk / mode->x.total, \
	             mode->dclk / (mode->x.total*mode->y.total)));

	save = *mode;		/* NOTE: we depend on the init proc in the    */
	*cmd = CMD_LOWER;	/* kernel driver!!                            */
	return EOK;

     case CMD_LOWER:
	tmp = save; save = *mode; *mode = tmp;	/* swap *mode and save	*/
	*cmd  = CMD_RAISE;
	return EOK;

     case CMD_RAISE:
	if (abs(save.dclk - timing[i].dclk) < abs(mode->dclk - timing[i].dclk))
	  *mode = save;	/* take timing which's dclk is closer to the preset */

	memset(&save, 0, sizeof(save));		/* clear saved mode	*/
	*cmd = CMD_CHECK;
	return EOK;

     case CMD_CHECK:
	if ((mode->dclk < monitor.dclk.min) || (monitor.dclk.max < mode->dclk))
	  {
	     TRACE(printk("%s: dclk of %i Hz is out of bounds.\n",\
	                  __FILE__, mode->dclk));
	     return -EMONITOR;
	  }

	if (   (monitor.maxdots.x < (int)mode->x.width) 
	    || (monitor.maxdots.y < (int)mode->y.width))
	  {
	     TRACE(printk("%s: resolution of %ix%i pixels is too high.\n", \
	                  __FILE__, mode->x.width, mode->y.width));
	     return -EMONITOR;
	  }

	if (! (mode->x.total && mode->y.total)) return -EMONITOR;
	hfreq = mode->dclk / mode->x.total;
	vfreq = hfreq / mode->y.total;
	if (   (abs(mode->dclk - timing[i].dclk) > (2*timing[i].dclk/100))
	    || (hfreq < monitor.hfreq.min) || (monitor.hfreq.max < hfreq)
	    || (vfreq < monitor.vfreq.min) || (monitor.vfreq.max < vfreq))
	  {
	    TRACE(printk("%s: timing limits violated.\n", __FILE__));
	    return -EMONITOR;
	  }
	
	h = (mode->request.graphtype == GT_TEXT) 
	      ? (mode->request.textgrid.x)
	      : (mode->request.textgrid.x ? mode->request.textgrid.x : 1);

	#define	ERR(X) (   (mode->y.X != timing[i].y.X)  \
			|| (abs(mode->x.X - timing[i].x.X) > h))
	if (   ERR(width)   || ERR(blankstart) || ERR(syncstart) 
	    || ERR(syncend) || ERR(blankend)   || ERR(total))
	  {
	    TRACE(printk("%s: error in timings.\n", __FILE__));
	    return -EMONITOR;
	  }
	#undef ERR

	*cmd = CMD_READY;
	return EOK;

     default:
	INTERNALERROR;
	return -EMONITOR;
   }

  #undef VISX
  #undef VISY
}

void
kgi_monitor_set_mode(struct kgi_mode *TM)
{
}

int
kgi_monitor_ioctl (struct inode *inode, struct file *file, unsigned int cmd,
		  unsigned long arg)
{
  int i, result;

  switch(cmd)
   {
     case MONITOR_GETINFO:
	_out_buf = &monitor; _io_cnt = sizeof(monitor);
	return EOK;

     case MONITOR_SETINFO:
	if (! suser())
	  return -EPERM;
	memcpy(&monitor, _io_buf, sizeof(monitor));
	return EOK;

     case MONITOR_GETNUMPRESET:
	return MAX_TIMINGS;

     case MONITOR_GETFREEPRESET:
	for (i = result = 0; i < MAX_TIMINGS; i++)
	 result += timing[i].dclk ? 0 : 1;
	return result;

     case MONITOR_SETPRESETMODE:
	if (! suser())
	  return -EPERM;
	return set_timing((struct kgi_preset *) _io_buf);

     case MONITOR_GETPRESETMODE:
	return get_timing((struct kgi_preset *) _io_buf);

     case MONITOR_CLEARPRESET:
	return clr_timing((struct kgi_preset *) _io_buf);

     default:
	return -EMONITOR;
   }
}
