/*\
|*| --------------------------------------------------------------------------
|*|	S3 SDAC clock synthesizer/ramdac driver - ramdac part 
|*| --------------------------------------------------------------------------
|*|
|*|	Copyright (C) 1995, The SCRDRV project.
|*|
|*|	This program is free software; you can redistribute it and/or modify
|*|	it under the terms of the GNU General Public License as published by
|*|	the Free Software Foundation; either version 2, or (at your option)
|*|	any later version.
|*|
|*|	You should have received a copy of the GNU General Public License
|*|	along with this program; see the file COPYING.  If not, write to
|*|	the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
|*|
|*| ---------------------------------------------------------------------------
\*/

#define	MAINTAINER	"Steffen Seeger, [seeger@physik.tu-chemnitz.de]"

#ifdef DEBUG
#define	NEED_GRAPHTYPE_SYMS
#endif

#include "graphdev.h"
#include "kgi.h"
#include "S3/sdac.h"

#include "IBM/clut.inc"

#define DAC_VERSION "0.00 ALPHA"

#if !((MONITORVERSION == 01) && (MODEVERSION == 03) && (RAMDACVERSION == 01))
  #warning kgi datastructures have changed, check this code !!
#endif

#define	SUPPORTED (GM_TEXT | GM_1BIT | GM_4BIT | GM_8BIT \
		  | GM_15BIT | GM_16BIT | GM_32BIT)

static struct kgi_ramdac ramdac =
       { "S3 Incorporated",		/* manufacturer		*/
	 "86C716-ME/MG SDAC",		/* model		*/
	 GM_TEXT | GM_4BIT | GM_8BIT |	/* supported modes	*/
	 GM_15BIT | GM_16BIT | GM_24BIT | GM_32BIT,
	 FL_DAC_8BIT,			/* flags		*/
	 { 0x7FFF, 0x7FFF },		/* maximum resolution	*/

	 DAC_8BIT_PB | 			/* device type		*/
	 DAC_16BIT_PB,
	 {        0, 110000000 },	/* normal mode		*/
	 { 67500000, 135000000 },	/* 2xclk mode		*/

	 {{ 	   0, 110000000 },	/* 	GT_TEXT		*/
	 {	   0,         0 },	/* 	GT_1BIT		*/
	 {	   0, 110000000 },	/* 	GT_4BIT		*/
	 {	   0, 110000000 },	/* 	GT_8BIT		*/	
	 {	   0, 110000000 },	/* 	GT_15BIT	*/
	 { 	   0, 110000000 },	/* 	GT_16BIT	*/
	 {	   0,  73000000 },	/* 	GT_24BIT	*/
	 {	   0,  55000000 }}	/* 	GT_32BIT	*/
	};

/*\
|*|	GGI interface
\*/

int
kgi_ramdac_init(void)
{
  TRACE(printk("  %s %s ramdac driver.\n", ramdac.manufact, ramdac.model));

  return EOK;
}

void
kgi_ramdac_done(void)
{
  TRACE(printk("%s %s driver removed.\n", ramdac.manufact, ramdac.model));
}

struct kgi_ramdac
*kgi_ramdac_get_info (void)
{
  return &ramdac; 
}

#if 0

int
kgi_ramdac_check_mode (struct kgi_mode *mode, enum kgi_tm_cmd cmd)
{
  int div;
  int newclock = mode->dclk;
  
  switch(mode->request.graphtype)
   {
     case GT_TEXT:	div =  1; break;
     case GT_1BIT:	div = 32; break;
     case GT_4BIT:	div =  8; break;
     case GT_8BIT:	div =  4; break;
  
     case GT_15BIT:
     case GT_16BIT:	div =  2; break;
  
     case GT_32BIT:	div =  1; break;
  
     case GT_24BIT:	return ERAMDAC;
     default:
  	TRACE(printk("%s: could not handle graphtype %i\n",\
  	     __FILE__, mode->request.graphtype));
 	return -ERAMDAC;
   }

  switch (cmd)
   {
     case CMD_PROPOSE:
	mode->lclk.mul = mode->clk.mul = mode->clk.div = 1;
        mode->lclk.div = div;
	TRACE(printk("%s: %i:%i multiplexing mode.\n", \
		     __FILE__, mode->lclk.div, mode->lclk.mul));

	mode->flags &= ~FL_DAC_2XCLOCK;
	mode->flags |= FL_DAC_8BIT;
	if((mode->request.graphtype == GT_32BIT) || 
	   (mode->request.graphtype == GT_15BIT) ||
	   (mode->request.graphtype == GT_16BIT))
	 mode->flags |= FL_DAC_GAMMA;

	return EOK;

     case CMD_LOWER:
	if (   (newclock < dac_params->clock[mode->request.graphtype].min)
	    || (newclock < dac_params->normal.min))
	  {
	    TRACE(printk("%s: DCLK too low (%i Hz).\n", __FILE__, newclock));
	    return -ERAMDAC;
	  }

	if (dac_params->clock[mode->request.graphtype].max < newclock)
    	  newclock = dac_params->clock[mode->request.graphtype].max;

	if (dac_params->normal.max < newclock)
	  {
	    TRACE(printk("%s: clock doubling mode not implemented yet.\n",\
			 __FILE__));
	    newclock = dac_params->normal.max;
	  }

	mode->dclk   = newclock;
	return EOK;

     case CMD_RAISE:
	if (   (newclock > dac_params->clock[mode->request.graphtype].max)
            || (newclock > dac_params->normal.max))
	  {
	    TRACE(printk("%s: DCLK too high (%i Hz).\n", __FILE__, newclock));
	    return -ERAMDAC;
	  }

	if (newclock < dac_params->clock[mode->request.graphtype].min)
	  newclock = dac_params->clock[mode->request.graphtype].min;

	if (newclock < dac_params->normal.min)
	  newclock = dac_params->normal.min;

	mode->dclk = newclock;
	return EOK;

     case CMD_CHECK:
	if (   (mode->lclk.mul != 1) || (mode->lclk.div != div)
	    || (mode->clk.mul  != 1) || (mode->clk.div !=    1))
	  return -ERAMDAC;

	if (   (mode->dclk < dac_params->normal.min)
	    || (mode->dclk < dac_params->clock[mode->request.graphtype].min))
	  return -ERAMDAC;

	if (   (mode->dclk > dac_params->normal.max)
	    || (mode->dclk > dac_params->clock[mode->request.graphtype].max))
	  return -ERAMDAC;

	return EOK;

    default:
	INTERNALERROR;
	return -ERAMDAC;
  }
}

void
kgi_ramdac_set_mode(struct kgi_mode *mode)
{
  struct kgi_monitor *monitor;

  /*\
  |*|	setup CR0
  \*/
     CR0 = DAC_C0_8BIT;		/* we'll always use 8 bit DAC */
     monitor = kgi_monitor_get_info();
     if (monitor->synctype & SYNC_ON_RED)     CR0 |= DAC_C0_SYNC_ON_RED;
     if (monitor->synctype & SYNC_ON_GREEN)   CR0 |= DAC_C0_SYNC_ON_GREEN;
     if (monitor->synctype & SYNC_ON_BLUE)    CR0 |= DAC_C0_SYNC_ON_BLUE;
     if (monitor->synctype & BLANK_PEDESTRAL) CR0 |= DAC_C0_BLANK_LEVEL;

  /*\
  |*|	setup CR1
  \*/
     switch(mode->request.graphtype)
      {
        case GT_TEXT:	CR1 = 0;		break;
        case GT_1BIT:	CR1 = DAC_C1_1BPP;	break;
	case GT_4BIT:	CR1 = DAC_C1_4BPP;	break;
	case GT_8BIT:	CR1 = DAC_C1_8BPP;	break;
	case GT_15BIT:	CR1 = DAC_C1_15BPP;	break;
	case GT_16BIT:	CR1 = DAC_C1_16BPP;	break;
	case GT_32BIT:	CR1 = DAC_C1_24BPP;	break;
	default:
	  INTERNALERROR;	/* for safety, and to trick gcc */
      };
     TRACE(printk("%s: graphtype = %s.\n", \
	    __FILE__, GRAPHTYPE(mode->request.graphtype)));

     /* CR1 |= DAC_C1_CLUT_BYPASS; turn on for true-color debugging only. */

  /*\
  |*|	setup CR2
  \*/
      CR2 =  ((mode->request.graphtype == GT_TEXT) ? 0:  DAC_C2_ENHANCED_MODE)
           | ((mode->flags & FL_MON_INTERLACE)    ? DAC_C2_INTERLACED : 0)
           | PCLK;

  /*\
  |*|	setup CR3
  \*/
     CR3 = 0;		/* we have no clock doubling yet ... */

  /*\
  |*|	program CR3,CR2,CR1,CR0 (CR3 looks a bit strange, but is correct ;-)
  \*/
     outbDAC(CR0 | DAC_C0_EXT_UNLOCK | DAC_C0_SLEEP_MODE, DAC_CONTROL0);
     outbDAC(DAC_PW_CR3_INDEX, DAC_PW_INDEX);
     outbDAC(CR3, DAC_STATUSREG);

     outbDAC(CR2, DAC_CONTROL2);
     outbDAC(CR1, DAC_CONTROL1);
     outbDAC(CR0, DAC_CONTROL0);

  set_CLUT_entries(mode->palette, 0, 256);	/* set clut of mode */
}

#endif

int
kgi_ramdac_ioctl (struct inode *inode, struct file *file, unsigned int cmd,
		  unsigned long arg)
{
  /* this may set
       -  dac_params (take care if 504 or 505 was detected).
       -  hardware cursor
       -  CLUT (or gamma correction table).
   */
  return -ERAMDAC;
}
