/*\
|*| ---------------------------------------------------------------------------
|*|	database conversion tool
|*| ---------------------------------------------------------------------------
|*|	This little program will convert a monitors.db entry into C code
|*|	and vice versa. At this moment only conversion into C is implemented.
|*| ---------------------------------------------------------------------------
|*|
|*|	Copyright (C) 1995, Steffen Seeger, [seeger@physik.tu-chemnitz.de]
|*|
|*|	This program is free software; you can redistribute it and/or modify
|*|	it under the terms of the GNU General Public License as published by
|*|	the Free Software Foundation; either version 2, or (at your option)
|*|	any later version.
|*|
|*|	You should have received a copy of the GNU General Public License
|*|	along with this program; see the file COPYING.  If not, write to
|*|	the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
|*|
|*| ---------------------------------------------------------------------------
\*/

#define	MAINTAINER	"Steffen Seeger, [seeger@physik.tu-chemnitz.de]"

#undef __KERNEL__

#define	NEED_MONITOR_SYMS
#define	NEED_GRAPHTYPE_SYMS

#include <stdio.h>
#include <scrdrv/graphdev.h>

char buf[256];

char
*modes(int m)
{
  int i, printed = 0; char *b = buf;

  if (m == GM_ALL)
    b += sprintf(b, "GM_ALL");
   else 
    for ( i = 0; i < GT_LAST; i++)
     if (m & (1 << i))
       b += sprintf(b, printed++ ? " | %s" : "%s", GRAPHMASK(i));

  return buf;
} 

#define	SYNC_SYMS 6
char *sync_sym[SYNC_SYMS] =
{
	"SYNC_ON_RED",  "SYNC_ON_GREEN", "SYNC_ON_BLUE", "BLANK_PEDESTRAL", 
        "POWER_SAVE",   "MULTISYNC"
};

char 
*syncsym(int s)
{
  int i, printed = 0; char *b = buf; 

  if ((s & COMPOSITE_SYNC) == SYNC_NORMAL) 
    printed++, b += sprintf(b, "SYNC_NORMAL");

  for (i = 0; i < SYNC_SYMS; i++)
   if (s & (1 << i))
     b += sprintf(b, printed++ ? " | %s" : "%s", sync_sym[i]);

  return buf;
} 

#define	FLAG_SYMS 32
char *flag_sym[FLAG_SYMS] =
{
  "FL_MON_INTERLACE", "FL_MON_POWERSAVE", "FL_MON_PROPSIZE", NULL,
  NULL, NULL, NULL, NULL, 

  "FL_DAC_8BIT", "FL_DAC_2XCLOCK", "FL_DAC_GAMMA", NULL,
  NULL, NULL, NULL, NULL,

  NULL, NULL, NULL, NULL,
  NULL, NULL, NULL, NULL,

  "FL_CHIP_LMAP", "FL_CHIP_DOUBLESCAN", "FL_CHIP_ACCEL", NULL,
  NULL, NULL, NULL, NULL
};

char 
*flagsym(int s)
{
  int i, printed = 0; char *b = buf; 

  for (i = 0; i < FLAG_SYMS; i++)
   if (s & (1 << i))
     b += sprintf(b, printed++ ? " | %s" : "%s", flag_sym[i]);

  return buf;
} 

void
fprintf_source(FILE *f, struct kgi_monitor *m)
{
  fprintf(f,"/*\\\n"
  "|*|\tNOTE: THIS FILE WAS GENERATED AUTOMATICALLY AND SHOULD NOT BE EDITED!\n"
  "\\*/\n\n#include \"graphdev.h\"\n#include \"kgi.h\"\n"
  "\nstruct kgi_monitor\n");
  
  fprintf(f,"monitor = {\n\t\"%s\", \"%s\",\n\t%s,\n",
  m->manufact, m->model, modes(m->modes));
 
  fprintf(f,"\t%s,\n\t{ %5i, %5i },\n\n",
  flagsym(m->flags), m->maxdots.x, m->maxdots.y);

  fprintf(f, 
  "\t{ %5i, %5i },\n\t%s | %s,\n\t%s,\n\t{ %9i, %9i },\n\t{ %9i, %9i },\n"
  "\t{ %9i, %9i }\n};\n",
  m->size.x, m->size.y, (m->disptype & MONO_DISPLAY) 
				? "MONO_DISPLAY" : "COLOR_DISPLAY", 
  DISPLAYTYPE(m->disptype), syncsym(m->synctype), 
  m->dclk.min, m->dclk.max, m->hfreq.min, m->hfreq.max,
  m->vfreq.min, m->vfreq.max);
}

void
fscanf_source(FILE *f, struct kgi_monitor *m)
{
  char dummy[256]; 

  if (2 != fscanf(f, " \"%63[^\"]\" , \"%63[^\"]\" , ", 
            m->manufact, m->model))
     goto error;

  if (2 != fscanf(f, " %256[^,] , %256[^,] , ", dummy, dummy ))
     goto error;

  if (4 != fscanf(f, " { %hu , %hu } , { %hu , %hu } , ", 
                  &m->maxdots.x, &m->maxdots.y, &m->size.x, &m->size.y))
    goto error;

  if (2 != fscanf(f, " %255[^,] , %255[^,] , ", dummy, dummy ))
     goto error;
   
  if (6 == fscanf(f, " { %i , %i } , { %i , %i } , { %i , %i } ",
     	 &m->dclk.min, &m->dclk.max, &m->hfreq.min, &m->hfreq.max, 
         &m->vfreq.min, &m->vfreq.max))
    return;

  error:
   perror("parse error.\n");
   exit (-1);
}

void
fprintf_entry(FILE *f, struct kgi_monitor *m)
{
  fprintf(f,"\"%s\" \"%s\"\t%ix%i\t%ix%i\t%i..%i %i..%i %i..%i"
         "\t0x%.8X 0x%.8X 0x%.8X 0x%.8X\n",
  m->manufact, m->model, m->maxdots.x, m->maxdots.y, m->size.x, m->size.y,
  m->dclk.min, m->dclk.max, m->hfreq.min, m->hfreq.max, 
  m->vfreq.min, m->vfreq.max, 
  m->modes, m->flags, m->synctype, m->disptype);
}

void fscanf_entry(FILE *f, struct kgi_monitor *m)
{
  int flags, sync, disp, 
  args =
  fscanf(f, "\"%63[^\"]\" \"%63[^\"]\" %hu x %hu %hu x %hu %u .. %u "
            " %u .. %u %u .. %u %i %i %i %i\n",
  m->manufact, m->model, &m->maxdots.x, &m->maxdots.y, &m->size.x, &m->size.y,
  &m->dclk.min, &m->dclk.max, &m->hfreq.min, &m->hfreq.max, 
  &m->vfreq.min, &m->vfreq.max, 
  (int *) &m->modes, &flags, &sync, &disp);

  if (args != 16)
    {
      perror("parse error\n"); exit(-2);
    }

  m->flags = flags; m->synctype = sync; m->disptype = disp;
}

int
main(int argc, char *argv[])
{
  struct kgi_monitor monitor;

  if (argc > 1)
    {
      if (!strcmp(argv[1], "--make-source"))
        {
          while (! feof(stdin))
           {
             fscanf_entry(stdin, &monitor);
             fprintf_source(stdout, &monitor);
           };
          return 0;
        }

      if (!strcmp(argv[1], "--make-entry"))
        {
          while (! feof(stdin))
           {
             fscanf_source(stdin, &monitor);
             fprintf_entry(stdout, &monitor);
           };
          return 0; 
        }
    }

  printf("usage: dbconv {--make-entry, --make-source}\n");

  return 0;
}

