/*\
|*| --------------------------------------------------------------------------
|*|	TI TLC34076 ramdac register definitions.
|*| --------------------------------------------------------------------------
|*|
|*|	Copyright (C) 1995 by Steffen Seeger, [seeger@physik.tu-chemnitz.de]
|*|
|*|	This program is free software; you can redistribute it and/or modify
|*|	it under the terms of the GNU General Public License as published by
|*|	the Free Software Foundation; either version 2, or (at your option)
|*|	any later version.
|*|
|*|	You should have received a copy of the GNU General Public License
|*|	along with this program; see the file COPYING.  If not, write to
|*|	the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
|*|
|*| ---------------------------------------------------------------------------
\*/

#ifndef _TLC34076_H
#define	_TLC34076_H

#include "IBM/vga_dac.h"

/* 0x05 ... 0x07 are reserved */

#define	DAC_CONTROL	0x08		/* general control register	    */
	#define	DAC_C_INV_HSYNC	0x01	/* invert hsync */
	#define	DAC_C_INV_VSYNC	0x02	/* invert vsync */
	#define	DAC_C_ENB_SPLIT	0x04	/* enable VRAM split shift reg */
	#define	DAC_C_ENB_SNM	0x08	/* enable special nibble mode  */
	#define	DAC_C_IRE_7_5	0x10	/* enable 7.5 IRE blank level  */
	#define	DAC_C_ENB_SYNC	0x20	/* enable sync  */
	#define	DAC_C_BIGENDIAN	0x40	/* little/big endian           */
	#define	DAC_C_MUXOUT	0x80	/* drive !MUXOUT high          */

#define	DAC_INPUTCLOCK	0x09		/* input clock selection register   */
	#define	DAC_I_CLOCK0	0x00	/* select clock0 */
	#define	DAC_I_CLOCK1	0x01	/* select clock1 */
	#define	DAC_I_CLOCK2	0x02	/* select clock2 */
	#define	DAC_I_CLOCK3	0x03	/* select clock3 */
	#define	DAC_I_CLK3_TTL	0x04	/* clock3 from TTL input */
	#define	DAC_I_CLK3_ECL	0x08	/* clock3 from ECL input */

#define	DAC_OUTPUTCLOCK	0x0A		/* output clock selection register  */
	#define	DAC_O_SCLK_1	0x00	/* SCLK = DCLK    */
	#define	DAC_O_SCLK_2	0x01	/* SCLK = DCLK/2  */
	#define	DAC_O_SCLK_4	0x02	/* SCLK = DCLK/4  */
	#define	DAC_O_SCLK_8	0x03	/* SCLK = DCLK/8  */
	#define	DAC_O_SCLK_16	0x04	/* SCLK = DCLK/16 */
	#define	DAC_O_SCLK_32	0x05	/* SCLK = DCLK/32 */
	#define	DAC_O_SCLK_LOW	0x07	/* drive SCLK low */

	#define	DAC_O_VCLK_1	0x00	/* VCLK = DCLK    */
	#define	DAC_O_VCLK_2	0x08	/* VCLK = DCLK/2  */
	#define	DAC_O_VCLK_4	0x10	/* VCLK = DCLK/4  */
	#define	DAC_O_VCLK_8	0x18	/* VCLK = DCLK/8  */
	#define	DAC_O_VCLK_16	0x20	/* VCLK = DCLK/16 */
	#define	DAC_O_VCLK_32	0x28	/* VCLK = DCLK/32 */
	#define	DAC_O_VCLK_LOW	0x38	/* drive VCLK high */

#define	DAC_MUX_CONTROL	0x0B		/* multiplex control register       */
	#define	DAC_M_VGA	0x2D	/* VGA<0:7> 8bpp mode	*/

	#define	DAC_M_1BPP	0x10	
	#define	DAC_M_2BPP	0x14
	#define	DAC_M_4BPP	0x18
	#define	DAC_M_8BPP	0x1C
	#define	DAC_M_15BPP	0x08
	#define	DAC_M_16BPP	0x09
	#define	DAC_M_24BPP	0x0E

#define	DAC_PALETTEPAGE	0x0C		/* palette page register 	    */

/* #define DAC_RESERVED	0x0D */

#define	DAC_TEST	0x0E		/* test register		    */
	#define	DAC_T_PAL_RED	0x00	/* test color palette red value	    */
	#define	DAC_T_PAL_GREEN	0x01	/* test color palette green value   */
	#define	DAC_T_PAL_BLUE	0x02	/* test colot palette blue value    */
	#define	DAC_T_ID_CODE	0x03	/* get ID code                      */
	#define	DAC_T_1AC_REG	0x04	/* 1's accumulation red value	    */
	#define	DAC_T_1AC_GREEN	0x05	/* 1's accumulation green value     */
	#define	DAC_T_1AC_BLUE	0x06	/* 1's accumulation blue value      */
	#define	DAC_T_ANALOG	0x07	/* analog test			    */
	#define	DAC_T_SEL_RED	0x80	/* select red for analg test	    */
	#define	DAC_T_SEL_GREEN	0x40	/* select green for analog test     */
	#define	DAC_T_SEL_BLUE	0x20	/* select blue for analog test	    */
	#define	DAC_T_SEL_145mV	0x10	/* 145mV reference for analog test  */

	#define	DAC_T_RESULT	0x08	/* result of test		    */
	#define	TLC34076_ID	0x76	/* ID number to be read		    */

/* #define DAC_RESERVED	0x0F */

#endif /* _TLC34076_H */
