/*\
|*| --------------------------------------------------------------------------
|*|     GGI device file definitions
|*| --------------------------------------------------------------------------
|*|
|*|	Copyright (C) 1995 by Andreas Beck, [becka@hp.rz.uni-duesseldorf.de]
|*|			  and Steffen Seeger, [seeger@physik.tu-chemnitz.de]
|*|
|*|	This program is free software; you can redistribute it and/or modify
|*|	it under the terms of the GNU General Public License as published by
|*|	the Free Software Foundation; either version 2, or (at your option)
|*|	any later version.
|*|
|*|	You should have received a copy of the GNU General Public License
|*|	along with this program; see the file COPYING.  If not, write to
|*|	the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
|*|
|*| ---------------------------------------------------------------------------
\*/

/* MAINTAINER	linux-ggi@eskimo.com */

#ifndef _graphdev_h
#define _graphdev_h

#include <linux/fs.h>
#include <asm/ioctl.h>

#include <scrdrv/kgi.h>
#include <scrdrv/err.h>

/*\
|*|	Definitions for device-registration
\*/

#define	DEVICEFILE_MAJOR	60
#define DEVICEFILE_NAME		"graphics"
#define	DEVICEFILE_MAGIC	((GGI_VERSION << 8) | 0xAB)

/*\
|*|	The ioctl's
|*|
|*|	Bit coding is according to <asm/ioctl.h> now. The <type>
|*|	gives the subsystem and the <nr> the command.
|*|
|*| !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
|*| !!!	NOTE: DON NOT CHANGE WITHOUT CHANGING THE KERNEL DRIVER INTERFACE !!!
|*| !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
\*/ 

#define IOC_DRIVER		0x00	/* kernel driver	*/
#define IOC_MONITOR		0x01	/* monitor driver	*/
#define IOC_RAMDAC		0x02	/* ramdac driver	*/
#define IOC_CLOCK		0x03	/* clock driver		*/
#define IOC_CHIPSET		0x04	/* chipset driver	*/
#define IOC_GRAPHICS		0x05	/* graphic driver	*/
/*				0x06	   reserved		*/
/*				0x07	   reserved		*/

/*\
|*|	A writing ioctl() sends data to the kernel, a reading gets something
|*|	back. 
\*/

/*\
|*|	ioctls for the graphics device
\*/

#define	IOC(subs,nr)		  _IO(IOC_ ## subs, nr) 
#define	IOR(subs,nr,data)	 _IOR(IOC_ ## subs, nr, data)
#define	IOW(subs,nr,data)	 _IOW(IOC_ ## subs, nr, data)
#define	IRW(subs,nr,data)	_IOWR(IOC_ ## subs, nr, data)

/*\
|*|	driver
\*/

#define	DRIVER_GETINFO		IOR(DRIVER, 0x00, struct kgi_driver)
#define	DRIVER_SETINFO		IOW(DRIVER, 0x00, struct kgi_driver)

#define	DRIVER_TESTDRIVER	IOC(DRIVER, 0x10)
#define	DRIVER_TESTMODE		IRW(DRIVER, 0x11, ggi_mode)

#define	DRIVER_SETGRAPHMODE	IRW(DRIVER, 0x12, ggi_mode)
#define	DRIVER_GETGRAPHMODE	IOR(DRIVER, 0x13, ggi_mode)
#define	DRIVER_GRAPHICS		IOC(DRIVER, 0x14)

#define	DRIVER_SETTEXTMODE	IRW(DRIVER, 0x15, ggi_mode)
#define	DRIVER_GETTEXTMODE	IOR(DRIVER, 0x16, ggi_mode)
#define	DRIVER_TEXT		IOC(DRIVER, 0x17)

/*\
|*|	chipset
\*/

#define	CHIP_GETINFO		IOR(CHIP, 0x00, struct kgi_chipset)
#define	CHIP_SETINFO		IOW(CHIP, 0x00, struct kgi_chipset)

#define	CHIP_SETVISFRAME	IOW(CHIP, 0x10, ggi_coord)
#define	CHIP_GETVISFRAME	IOR(CHIP, 0x10, ggi_coord)
#define	CHIP_SETSPLITLINE	IOC(CHIP, 0x11)
#define	CHIP_GETSPLITLINE	IOC(CHIP, 0x12)

#define	CHIP_SETDEFFONT		IOW(CHIP, 0x13, ggi_bm_font)
#define	CHIP_GETDEFFONT		IRW(CHIP, 0x13, ggi_bm_font)
#define	CHIP_SETFONTCHAR	IOW(CHIP, 0x14, ggi_bm_char)
#define	CHIP_GETFONTCHAR	IRW(CHIP, 0x14, ggi_bm_char)

/*\
|*|	graphics
\*/


/*\
|*|	ramdac
\*/

#define	RAMDAC_GETINFO		IOR(RAMDAC, 0x00, struct kgi_ramdac)
#define	RAMDAC_SETINFO		IOW(RAMDAC, 0x00, struct kgi_ramdac)

#define	RAMDAC_SETCLUT		IOW(RAMDAC, 0x10, struct ggi_palette)
#define	RAMDAC_GETCLUT		IRW(RAMDAC, 0x10, struct ggi_palette)

/*\
|*|	monitor
\*/

#define	MONITOR_GETINFO		IOR(MONITOR, 0x00, struct kgi_monitor)
#define	MONITOR_SETINFO		IOW(MONITOR, 0x00, struct kgi_monitor)

#define	MONITOR_GETNUMPRESET	IOC(MONITOR, 0x10)
#define	MONITOR_GETFREEPRESET	IOC(MONITOR, 0x11)
#define	MONITOR_SETPRESETMODE	IOW(MONITOR, 0x12, struct kgi_preset)
#define	MONITOR_GETPRESETMODE	IRW(MONITOR, 0x12, struct kgi_preset)
#define	MONITOR_CLEARPRESET	IOW(MONITOR, 0x13, struct kgi_preset)


#define MMAP_LINEAR	0	/* Linear frame buffer */
#define MMAP_FRAME	1	/* banked frame buffer */


#ifdef __KERNEL__
/*\
|*|	global ioctl variables.
|*|
|*|	The data transfer scheme used is as follows:
|*|
|*|	  - The kernel driver checks if the IOC_IN flag in cmd is set. If so,
|*|	    the number of bytes to copy from arg is determined from cmd and
|*|	    copied to _io_buf. _out_buf is set to NULL.
|*|
|*|	  - Then kgi_subsystem_ioctl is invoked according to the subsystem 
|*|	    coded in cmd. This performs the action according to the request in
|*|	    cmd. If output is requested, _out_buf is set to the 'result' buffer
|*|	    and _io_cnt to the number of bytes to copy from there. Note that
|*|	    local variables will be out of scope after return. Use _io_buf then.
|*|
|*|	  - After kgi_subsystem_ioctl has returned, cmd is checked for the 
|*|	    IOC_OUT flag to be set. If so (and _out_buf and _io_cnt are valid),
|*|	    _io_cnt bytes are copied from _out_buf to arg. 
|*|	    
|*|	  - The value returned by kgi_subsystem_ioctl is returned.
\*/

#define	_IO_BUF_SIZE	(1 << _IOC_SIZEBITS)
extern	unsigned int  _io_cnt;	/* # of bytes to read from/write to *arg */
extern	unsigned char _io_buf[_IO_BUF_SIZE];	/* treat as uninitialized. */
extern	void *_out_buf;

#endif	/* #ifdef __KERNEL__ */
#endif	/* #ifdef _graphdev_h */

