/*\
|*| --------------------------------------------------------------------------
|*|	kernel module interface definitions
|*| --------------------------------------------------------------------------
|*|
|*|	Copyright (C) 1995 by Steffen Seeger   [seeger@physik.tu-chemnitz.de]
|*|			  and Andreas Beck   [becka@hp.rz.uni-duesseldorf.de]
|*|
|*|	This program is free software; you can redistribute it and/or modify
|*|	it under the terms of the GNU General Public License as published by
|*|	the Free Software Foundation; either version 2, or (at your option)
|*|	any later version.
|*|
|*|	You should have received a copy of the GNU General Public License
|*|	along with this program; see the file COPYING.  If not, write to
|*|	the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
|*|
|*| ---------------------------------------------------------------------------
\*/

/* MAINTAINER	linux-ggi@eskimo.com */

#ifndef _kgi_module_h
#define _kgi_module_h

#include <linux/kernel.h>
#include <linux/mm.h>

#include <scrdrv/ggi.h>

/*\
|*|	some useful debugging macros
\*/

#define INTERNALERROR printk("KGI: %s: %i: internal error, please report to " \
			MAINTAINER ".\n",__FILE__,__LINE__);

#ifdef __KERNEL__
 #define NOT_TESTED \
    printk(\
    "\n\nGood !! You hit untested code in %s line %i. Please report whether\n"\
    "there were any difficulties or not. (Simply send this message and a\n"\
    "short explanation of your configuration to " MAINTAINER ".\n\n\n",\
    __FILE__,__LINE__)
 #ifdef DEBUG
  #define TRACE(X) X
 #else
  #define TRACE(X)
 #endif
#endif

/*\
|*|	generalized hardware IO functions.
\*/

	void	outbDAC(unsigned char value, unsigned char dac_reg);
unsigned char	inbDAC(unsigned char dac_reg);
	void	outsbDAC(unsigned short dac_reg, void *buf, int count);
	void	insbDAC (unsigned short dac_reg, void *buf, int count);

	void	setCLK(int value);	/* drive CS[0:3] lines to value	*/
	int	getCLK(void);		/* read state of CS[0:3] lines	*/

#define	IS_CRITICAL	unsigned long __critical_flags
#define	ENTER_CRITICAL	save_flags(__critical_flags); cli()
#define	LEAVE_CRITICAL	restore_flags(__critical_flags)

/*\
|*|	kernel driver data structures
\*/

#if (GGIVERSION != 01)
  #warning	common ggi structures changed, check kgi structures !!
#endif

enum kgi_pointer_state
{
	PF_SHOWN	= 0x00000001,	/* pointer displayed	*/
	PF_FB_MODIFIED	= 0x00000002	/* framebuffer modified	*/
};

struct kgi_pointer
{
  short			x,y;		/* pointer position, (chars/pixels) */
  enum kgi_pointer_state state;		/* current state                    */
  void (*show)(struct kgi_pointer *, unsigned int);
  void (*hide)(struct kgi_pointer *, unsigned int);	
};

#define	COMMONVERSION	01

	enum kgi_flags
	{	FL_MONITOR	= 0x000000FF,
		FL_MON_INTERLACE  = 0x00000001,	/* Ain't that rather chipset ? */
		FL_MON_POWERSAVE  = 0x00000002,
		FL_MON_PROPSIZE   = 0x00000004,	/* size depends on mode */

		FL_DAC		= 0x0000FF00,
		FL_DAC_8BIT	= 0x000100,	/* 8 or 6 bit dac */
		FL_DAC_2XCLOCK	= 0x000200,	/* internal 2*clk */
		FL_DAC_GAMMA	= 0x00040000,	/* true color LUT */

		FL_CLOCK	= 0x00FF0000,

		FL_CHIPSET	= 0xFF000000,
		FL_CHIP_LMAP	   = 0x01000000,/* linear mapping */
		FL_CHIP_DOUBLESCAN = 0x02000000,
		FL_CHIP_ACCEL	   = 0x04000000,
		FL_CHIP_COLORTEXT  = 0x08000000
	};

	#define	KGI_COMMON char manufact[64],	/* manufacturer		*/\
				model[64];	/* model/trademark	*/\
	          ggi_graphmask	modes;		/* supported modes	*/\
	         enum kgi_flags	flags;		/* special capabilities	*/\
	              ggi_coord	maxdots;	/* maximum resolution	*/

	struct	kgi_common
	{
	  KGI_COMMON
	};


#define	MONITORVERSION	01

	enum	kgi_display
	{
		EGA_8 = 0,	/* EGA monitor (8 colors)		*/
		EGA_16,		/* EGA monitor (16 colors)		*/
		VGA,		/* analog VGA monitor			*/
		PLASMA,		/* plasma display used on laptop's (???)*/
		TFT,		/* (???)				*/
		LCD,		/* Liquid Crystal Display		*/
		HERCMONO,	/* Monochrome, 3 cols (off,on,hilight)  */
		LAST_DISPTYPE,	/* last defined type			*/
					/* COLOR and MONO are flags	*/
		COLOR_DISPLAY = 0,	/* able to display colors	*/
		MONO_DISPLAY  = 0x80000000 /* b&w or grayscale display	*/
	};

	#ifdef NEED_MONITOR_SYMS
 	  static char *_kgi_display_type[LAST_DISPTYPE] =
	   { "EGA_8", "EGA_16", "VGA", "PLASMA", "TFT", "LCD", "HERCMONO" };
  	  #define DISPLAYTYPE(X) (_kgi_display_type[(X) & ~MONO_DISPLAY])
	#endif

	enum	kgi_sync
	{
		SYNC_NORMAL 	= 0,	/* normal syncing via separate lines  */
		SYNC_ON_RED     = 1, 	/* send sync on r,g,b line. These can */
		SYNC_ON_GREEN   = 2,	/* be ORed together.		      */
		SYNC_ON_BLUE    = 4,
		COMPOSITE_SYNC	= 7,	/* mask to check for composite sync   */
		BLANK_PEDESTRAL = 8,
		POWER_SAVE	= 16,	/* has powersaveing via sync (VESA)   */
		MULTISYNC	= 32
	};

	struct kgi_monitor 
	{
	   KGI_COMMON

	   ggi_coord        size;	/* max. physical picture size in mm   */
	   enum kgi_display disptype;   /* display type			      */
	   enum kgi_sync    synctype;	/* synctype			      */
	   ggi_range        dclk;	/* pixel frequency range (Bandwidth)  */
	   ggi_range        hfreq;	/* horizontal frequency range	      */
	   ggi_range        vfreq;	/* vertical frequency range 	      */
	};


#define	CHIPSETVERSION	01

	struct kgi_chipset
	{
	  KGI_COMMON

	  int               ramsize;	/* installed RAM size	*/
	  ggi_range         dclk;	/* DCLK range		*/
	};


#define RAMDACVERSION	01

	enum kgi_dac_type
	{
	    DAC_8BIT_PB   = 0x01,	/*   8 bit pixel bus		*/
	    DAC_16BIT_PB  = 0x02,	/*  16 bit pixel bus		*/
	    DAC_24BIT_PB  = 0x04,	/*  24 bit pixel bus		*/
	    DAC_32BIT_PB  = 0x08,	/*  32 bit pixel bus		*/
	    DAC_64BIT_PB  = 0x10,	/*  64 bit pixel bus		*/
	    DAC_128BIT_PB = 0x20,	/* 128 bit pixel bus		*/
	    DAC_PB_MASK   = 0x3F,	/* pixel bus width mask		*/
	    DAC_VRAM      = 0x80000000	/* VRAM shift clock support	*/
	};

	struct kgi_ramdac
	{
	  KGI_COMMON

	  enum   kgi_dac_type	dactype;	/* general device type	     */
	  ggi_range		normal;		/* DCLK range (normal mode)  */
	  ggi_range		dblclk;		/* DCLK (internal doubling)  */
          ggi_range		clock[GT_LAST];	/* DCLK ranges		     */
	};
#if 0
	enum kgi_dacmode
	{ 
	  DAC_MUX_1	= 0,	/* DCLK range if  1:1 multiplex */
	  DAC_MUX_2	= 1,	/* DCLK range if  2:1 multiplex */ 
	  DAC_MUX_4	= 2,	/* DCLK range if  4:1 multiplex */
	  DAC_MUX_8	= 3,	/* DCLK range if  8:1 multiplex */
	  DAC_MUX_16   	= 4,	/* DCLK range if 16:1 multiplex */
	  DAC_MUX_32	= 5,	/* DCLK range if 32:1 multiplex */
	  DAC_MUX_DOUBLE= 8,	/* DCLK limit if internal clock doubling */
	  DAC_MUX_NORMAL= 9	/* DCLK limit if in normal mode */
	};
#endif

#define	CLOCKVERSION	01

	#define	MAX_FIXED_CLOCKS	16
	#define	MAX_PROG_CLOCKS		8

	enum kgi_clock_type
	{
	  CLK_TYPE_FIXED,	CLK_TYPE_PROG
	};

	struct kgi_clock
	{
	  KGI_COMMON

	  enum kgi_clock_type	type;
	  union
	  {
	     int	freq[MAX_FIXED_CLOCKS];
	     ggi_range	range[MAX_PROG_CLOCKS];		
	  } clock;
	};


#define	INFOVERSION	01

	union kgi_info		/* union to ensure proper mem allocation  */
	{
	  struct kgi_common	common;

	  struct kgi_monitor	monitor;
	  struct kgi_ramdac	ramdac;
	  struct kgi_clock	clock;
	  struct kgi_chipset	chipset;
	};


#define	MODEVERSION	03

	struct kgi_timing	/* timing parameters (in DCLK's or lines)  */
	{
	  unsigned short	width;		/* visible pixels/lines	*/
	  unsigned short	blankstart;	/* assertion of BLANK	*/
	  unsigned short	syncstart;	/* assertion of SYNC	*/
	  unsigned short	syncend;	/* deassertion of SYNC	*/
	  unsigned short	blankend;	/* deassertion of BLANK	*/
	  unsigned short	total;		/* total pixels/lines	*/
	  unsigned short	polarity;	/* sync polarity	*/
	};

	struct kgi_preset
	{
	  int			dclk;		/* dot clock in Hz       */
	  struct kgi_timing	x,y;		/* x and y timing values */
	};

	struct kgi_mode		/* mode setup info. (calculated or preset) */
	{
	  ggi_mode		request;	/* requested mode	      */

	  /* These will be auto-calculated by the various drivers */
	  struct kgi_timing	x,y;		/* x and y timing parameter   */
	  ggi_coord		magnify;	/* stretched ... times	      */
	  enum kgi_flags	flags;		/* special capabilities used  */
	  int			dclk;		/* dotclock (master reference)*/
	  ggi_ratio		clk;		/* rate programmed by clock   */
	  ggi_ratio		lclk;	/* transfer rate to dac (byte/s)      */

	  /* These are set by the kernel driver. */
	  ggi_palette		*palette; 	/* palette or gamma map	      */
	  ggi_font		*font0, *font1;	/* fonts for textmodes	      */
#ifdef 0
          enum kgi_dacmode	muxmode;
#endif

	  void (*set_origin)  (unsigned int);
	  void (*pointer_show)(struct kgi_pointer *, unsigned int);
	  void (*pointer_hide)(struct kgi_pointer *, unsigned int);
	  void (*cursor_show) (struct kgi_pointer *, unsigned int);
	  void (*cursor_hide) (struct kgi_pointer *, unsigned int);

	  int			monitor;	/* driver specific options    */
	  int			ramdac;		/* check_timing should assume */
	  int			clock;		/* them not valid.	      */
	  int			chipset;
	};

	enum kgi_tm_cmd
	{
	  CMD_PROPOSE,		/* propose a timing			*/
	  CMD_LOWER,		/* may modify, but only to lower clock	*/
	  CMD_RAISE,		/* may modify, but only to higher clock	*/
	  CMD_CHECK,		/* don't modify, but check if valid	*/
	  CMD_READY		/* don't care.	(should never happen)	*/
	};

/*\
|*|	Hardware driver interfaces. Stick to these functions when interfacing
|*|	drivers !
\*/

int	kgi_monitor_init (void);
void	kgi_monitor_done (void);
void	kgi_monitor_reset (void);
struct	kgi_monitor *kgi_monitor_get_info (void);
int	kgi_monitor_check_mode(struct kgi_mode *mode, enum kgi_tm_cmd *cmd);
void	kgi_monitor_set_mode (struct kgi_mode *mode);
int	kgi_monitor_ioctl (struct inode *inode, struct file *file,
			   unsigned int cmd, unsigned long arg);

int	kgi_clock_init (void);
void	kgi_clock_done (void);
void	kgi_clock_reset (void);
struct	kgi_clock *kgi_clock_get_info (void);
int	kgi_clock_check_mode (struct kgi_mode *mode, enum kgi_tm_cmd cmd);
void	kgi_clock_set_mode (struct kgi_mode *mode);
int	kgi_clock_ioctl (struct inode *inode, struct file *file,
			 unsigned int cmd, unsigned long arg);

int	kgi_chipset_init (void);
void	kgi_chipset_done (void);
void	kgi_chipset_reset (void);
struct	kgi_chipset *kgi_chipset_get_info (void);
int	kgi_chipset_check_mode (struct kgi_mode *mode, enum kgi_tm_cmd cmd);
void	kgi_chipset_set_mode (struct kgi_mode *mode);
int	kgi_chipset_ioctl (struct inode *inode, struct file *file,
			   unsigned int cmd, unsigned long arg);

int	kgi_ramdac_init (void);
void	kgi_ramdac_done (void);
void	kgi_ramdac_reset (void);
struct	kgi_ramdac *kgi_ramdac_get_info (void);
int	kgi_ramdac_check_mode (struct kgi_mode *mode, enum kgi_tm_cmd cmd);
void	kgi_ramdac_set_mode (struct kgi_mode *mode);
int	kgi_ramdac_ioctl (struct inode *inode, struct file *file,
			  unsigned int cmd, unsigned long arg);

int	kgi_graphics_ioctl (struct inode *inode, struct file *file,
			    unsigned int cmd, unsigned long arg);

/*\
|*|	some generic stuff for framebuffer mapping.
|*|	- variables have to be initialized by the chipset driver. 
|*|	- functions have to be provided by the kernel interface.
\*/

extern unsigned long	fb_graphics;  /* phys. addr. of graphics framebuffer  */
extern size_t   	fb_graph_len; /* length of the graphics framebuffer   */
extern unsigned long	fb_text;      /* physical address of text framebuffer */
extern size_t		fb_text_len;  /* length of the text framebuffer       */

unsigned long kgi_get_physical_mem(	/* search for free physical memory    */
		unsigned long start,	/* in the area from 'start'           */
		unsigned long end,	/* to 'end'			      */
		unsigned long align,	/* aligned on a multiple of 'align'   */
		size_t len,		/* and of size 'len'		      */
		char *device);		/* for 'device'			      */
void	kgi_free_physical_mem(unsigned long start, size_t len);

/*\
|*|	default variables
\*/

extern ggi_mode		default_textmode;
extern ggi_font		default_font;
extern ggi_palette	default_text_palette;
extern ggi_palette	default_graphics_palette;
extern ggi_palette	default_gamma_map;

#endif	/* _kgi_module_h */
